==> jungle-1.0.tar.gz: jungle-1.0/jungle/apps/timereport/time_display.py <==
import os
import sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'jobs.settings'
from jungle.timereport.models import Transit, TimetableProfile
from jungle.functions.utils import MatrixDict
from datetime import *
from django.utils.datastructures import SortedDict
from django.conf import settings
from jungle.registration.models import UserProfile
from django.utils.translation import gettext as _

HOLIDAYS = (
    ## FIXME: to be written seriously!!!! (from db)
#    date(2007, 4, 9),
    date(2007, 4, 25),
    date(2007, 5, 1),
    date(2007, 6, 2),
    date(2007, 8, 15),
    date(2007, 12, 8),
    date(2007, 12, 25),
    date(2007, 12, 26),
    date(2008, 1, 1),
    date(2008, 3, 25),
    date(2008, 6, 2),
    date(2008, 8, 15),
    date(2008, 12, 8),
    date(2008, 12, 25),
    date(2009, 1, 1),
    )
try:
    ## local holidays can be written in a local file
    from local_holiday import HOLIDAYS as LOCAL_HOLIDAYS
    HOLIDAYS = (HOLIDAYS + LOCAL_HOLIDAYS)
except ImportError:
    pass

if getattr(settings, 'TIMEREPORT_TOTAL_CLASS', False):
    from jungle.functions.conf import get_class
    Total = get_class(settings.TIMEREPORT_TOTAL_CLASS)
else:
    from total import Total
from total import PeriodTotal
class Day(object):
    def __init__(self, day):
        self.day = day
        self.cls = self.get_class()
        
    def is_holiday(self):
        if self.day.strftime('%w') in ('0','7'):
            return True
        elif self.day in HOLIDAYS:
            return True
        else:
            return False

    def get_class(self):
        if self.is_holiday():
            return "holiday"
        else:
            return ""

    def num(self):
        return self.day.strftime('%w')
    
    def next(self):
        return self.day + timedelta(days=1)

    def d(self):
        return str(self.day.strftime("%d/%m/%Y"))
    
    def __unicode__(self):
        return unicode(self.day.strftime("%d/%m/%Y - %a"))

    def __str__(self):
        return str(self.day.strftime("%d/%m/%Y - %a"))

class MyDict(SortedDict):
    """a SortedDict that creates on the fly empty SortedDict as elements if not
    declared and that allows a sum with a list of empty elements (promoting to list)
    """
    
    def __getitem__(self, name):
        if name not in self.keys():
            super(MyDict,self).__setitem__(name, self.__class__())
        return super(MyDict,self).__getitem__(name)

    def __add__(a, b):
        """don't complain if the value is empty, just return added value""" 
        if not a :
            return b
        return super(MyDict, self).__add__(a,b)

    def cls(self):
        return ''

    def to_html(self, user=None):
        return u''

    def __str__(self):
        if not self:
            return ''
        else:
            return super(MyDict, self).__str__()

    def __unicode__(self):
        if not self:
            return u''
        else:
            return super(MyDict, self).__unicode__()

    
class TransitSlot(object):
    """a class that holds info on transit: both real and missing
    It's not a Transit object in that it exists also when no transit is present
    and can hold more than 1 transit till the situation is not reviewd by users.
    Essentially it's the best representation of the single cell in the
    html table, so must be able to render a correct class, a and create links
    
    """

    
    def __init__(self, datetime, Id, profile_id, col, miss=False, blank=False, transit=None):
        self.datetime = [datetime]
        self.ids = {datetime:Id}
        self.profile_id = profile_id
        self.col = col
        self.transit = transit
        ## a holiday is a regular missing, don't show 'missing'
        self.miss = miss and (datetime.date() not in HOLIDAYS)
        self.blank = blank
        if self.miss and self.datetime[0] > datetime.now():
            self.miss = False
            self.blank=True
        self.extra_class = ''

    def __add__(a, b):
        ## Note! this only works if adding an element with 1 datetime!!!
        # self.datetime is a list!
        a.datetime.extend(b.datetime)
        a.ids[b.datetime[0]]  = b.ids[b.datetime[0]]
        return a

    def to_html(self, user_id): #__str__(self):
        #import pdb; pdb.set_trace()
        if self.miss:
            return self.link(self.datetime[0], user_id)
        elif self.blank:
            return u''
        else:
            return u" - ".join([self.link(x, user_id) for x in self.datetime])            

    def link(self, datetime, user_id):
        opts = 'top=200, left=200, width=580, height=320, status=no, menubar=no, toolbar=no scrollbar=no'
        try:
            info = {
                'user' : user_id,
                'odv'  : self.col,
                'date' : self.datetime[0].strftime('%Y/%m/%d')
                }
        except:
            #import pdb
            #pdb.set_trace()
            raise
        link = '/timereport/transit/add/%(user)s/%(odv)s/%(date)s/?mode=short' % (info)
        onclick = "window.open('%s', '', '%s'); return false;" % (link, opts)
        if self.miss:
            return '<a href="" onclick="%s">%s</a>' % (onclick, _('Missing'))
        else:
            return """<a href="" onclick="window.open('/timereport/transit/edit/%s/', '', '%s'); return false;">%s</a>""" % (
                self.ids[datetime], opts.replace('320', '520') , datetime.strftime('%H:%M'))

    def cls(self):
        #import pdb; pdb.set_trace()
        if self.miss or len(self.datetime) > 1:
            return 'error'
        elif self.blank:
            return ''
        else:
            if self.transit and self.transit.reason.column == 'illness':
                return self.classes("illness")

            if self.transit and self.transit.reason.worked != 'w':
                if self.transit.approved == 2:
                    return self.classes("vacation-approved")
                else:
                    return self.classes("vacation")

            if self.transit and self.transit.status == 2:
                return self.classes("manual")

            if self.datetime[0].date() in HOLIDAYS:
                return self.classes("extra")
            try:
                self.official_time = get_official_time(self.profile_id, self.datetime[0], self.col)
            except:
                return self.classes("extra")

            try:
                delta = self.datetime[0] - datetime.combine(self.datetime[0], self.official_time) 
            except TypeError:
                #print "TypeError in ", self.profile_id, self.datetime[0].strftime("%w %a"), self.datetime[0], self.col
                return ''

            #import pdb; pdb.set_trace()
            delta_min = delta2min(delta)

            if self.col in (1,3):
                if delta_min < 0: ## before official time
                    if delta_min < -ProfileDeltas[self.profile_id][0] : 
                        return self.classes('longer')
                    else:
                        return ''
                else: ## late
                    if delta_min > ProfileDeltas[self.profile_id][1] : 
                        return self.classes('shorter')
                    else:
                        return ''

            if self.col in (2,4):
                if delta_min <  0: ## before official time
                    if delta_min < -ProfileDeltas[self.profile_id][0] : 
                        return self.classes('shorter')
                    else:
                        return self.classes('')
                else: 
                    if delta_min > ProfileDeltas[self.profile_id][1] : 
                        return self.classes('longer')
                    else:
                        return self.classes('')

    def classes(self, html_class):
        return "%s %s" % (self.extra_class, html_class)

    def tip(self):
        if self.transit:
            if self.transit.iter_id:
                self.extra_class = 'help_text'
                return 'title="%s"' % Notes[self.transit.iter_id].description
        return ''
    def __repr__(self):
        return "<TransitSlot: %s (%s)>" % ([d.strftime('%x %X') for d in self.datetime], self.transit)

def get_official_time(profile_id, date, col):
    # detail has: time + duration!
    details = ProfileDayDetails[profile_id][date.isoweekday()]
    if col % 2:
        return details[col].time
    else:
        return details[col-1].time_out
                    
def get_official_datetime(profile_id, day, col):
    date = day.day
    time = get_official_time(profile_id, date, col)
    return datetime.combine(date, time)

def get_transits(filter={}, queryset=None):
    """
    return a tuple (transits, days, users, Totals, PeriodTotals). Each is a dict
    suitable for looping ie:
    days: all days are present in the required range, even days in which
          no transit is registered
    transits: every cell has a TransitSlot object so that can be used
          easily in templates w/o checking existance
    """
    global Notes
    T = MyDict()
    Users = MyDict()
    Days = []
    Totals = MyDict()
    Notes =  {}  # id of the iter is the key, the value is the iter
    PeriodTotals = MyDict()

    queryset = queryset.filter(**filter).select_related()
    for t in  queryset.order_by('user', 'date_reference', 'reason', 'date_time'):
        day = t.date_reference
        Days += [day]
        T[t.user_id][day][t.odv] +=  TransitSlot(
             t.date_time, t.id, t.profile.id, t.odv, transit=t)
        if t.iter_id:
            if t.iter_id not in Notes:
                Notes[t.iter_id] = t.iter

    ## fill in data on users
    query = UserProfile.objects.filter(user__in=T.keys())
    for u in query.order_by('badge_num'):
        Users[u.user_id]['user'] = u.user
        Users[u.user_id]['profile_id'] = u.time_profile.id
            
    ## check all entries needed as per profile are present else add
    ## a TransitSlot that will translate to Alert / Missing
    DayList = all_days_as_list(filter, Days, )
    for user_id in T.keys():

        ## the following profile_id will be overwritten by the first
        ## transit, but is here in case the first day the user is missing
        profile_id = Users[user_id]['profile_id']
        period_total = PeriodTotal(user_id,)
        PeriodTotals[user_id] = period_total
        
        for day in DayList:

            total = Total(day, ProfileDayDetails[profile_id][day.day.isoweekday()])
            Totals[user_id][day.day] = total
            for col in (1,2,3,4,11,12):

                if T[user_id][day.day][col]:  

                    total.add(T[user_id][day.day][col].transit)
                    profile_id = T[user_id][day.day][col].profile_id
                    
                else :
                    add_blank_or_missing_transit(T, profile_id, user_id, day, col)

            ## NOTE: period_total.add triggers the calculus of time!!
            period_total.add(total)
    return ( T, DayList, Users, Totals, PeriodTotals)

def add_blank_or_missing_transit(T, profile_id, user_id, day, col):
    # no entry for this user/day/col, maybe just col 11, 12...
    # this is ok but I want to define a TransitSlot instance just
    # for template usage.

    blank = False
    miss  = False
    try:
        ## next will fail if no official datetime is due
        date_time = get_official_datetime(profile_id, day, col)

        miss = True
        if day.is_holiday():
            blank=True

    except AttributeError, e:  # 'MyDict' object has no attribute 'time'
        ## it's ok not to have anything
        date_time = datetime.combine(day.day, time(0,0))
        blank = True

    T[user_id][day.day][col] = TransitSlot(
        date_time, None, profile_id, col, miss=miss, blank=blank)

    
def all_days_as_list(filter, days):
    """if filter has date_time__gte / date_time__lte use those days
    else look in days to get the start/end period
    """
    #import pdb; pdb.set_trace()
    #from jungle.forms.fields import JungleDateField
    #f = JungleDateField
    if 'date_reference__gte' in filter:
        start = filter['date_reference__gte']
    else:
        try:
            start = days[0]
        except IndexError:
            return []
        
    if 'date_reference__lte' in filter:
        end = filter['date_reference__lte']
    else:
        try:
            end = days[-1]
        except IndexError:
            end = date.today()
        
    if start == end:
        return [Day(start)]
    if start > end: # prevent infinite looping
        return []

    D = [Day(start)]
    last = Day(start)

    while True:
        last = Day(last.next())
        D += [last]
        if last.day == end:
            return D

def delta2min(delta):
    seconds = delta.days*86400+delta.seconds
    return seconds/60

ProfileDayDetails = TimetableProfile.get_details()
ProfileDeltas  = TimetableProfile.get_deltas()

==> jungle-1.0.1.tar.gz: jungle-1.0.1/jungle/apps/timereport/time_display.py <==
import os
import sys
os.environ['DJANGO_SETTINGS_MODULE'] = 'jobs.settings'
from jungle.timereport.models import Transit, TimetableProfile
from jungle.functions.utils import MatrixDict
from datetime import *
from django.utils.datastructures import SortedDict
from django.conf import settings
from jungle.registration.models import UserProfile
from django.utils.translation import gettext as _

HOLIDAYS = (
    ## FIXME: to be written seriously!!!! (from db)
    #    date(2007, 4, 9),
    date(2007, 4, 25),
    date(2007, 5, 1),
    date(2007, 6, 2),
    date(2007, 8, 15),
    date(2007, 12, 8),
    date(2007, 12, 25),
    date(2007, 12, 26),
    date(2008, 1, 1),
    date(2008, 3, 25),
    date(2008, 6, 2),
    date(2008, 8, 15),
    date(2008, 12, 8),
    date(2008, 12, 25),
    date(2009, 1, 1),
)
try:
    ## local holidays can be written in a local file
    from local_holiday import HOLIDAYS as LOCAL_HOLIDAYS
    HOLIDAYS = (HOLIDAYS + LOCAL_HOLIDAYS)
except ImportError:
    pass

if getattr(settings, 'TIMEREPORT_TOTAL_CLASS', False):
    from jungle.functions.conf import get_class
    Total = get_class(settings.TIMEREPORT_TOTAL_CLASS)
else:
    from total import Total
from total import PeriodTotal
class Day(object):

    def __init__(self, day):
        self.day = day
        self.cls = self.get_class()

    def is_holiday(self):
        if self.day.strftime('%w') in ('0', '7'):
            return True
        elif self.day in HOLIDAYS:
            return True
        else:
            return False

    def get_class(self):
        if self.is_holiday():
            return "holiday"
        else:
            return ""

    def num(self):
        return self.day.strftime('%w')

    def next(self):
        return self.day + timedelta(days=1)

    def d(self):
        return str(self.day.strftime("%d/%m/%Y"))

    def __unicode__(self):
        return unicode(self.day.strftime("%d/%m/%Y - %a"))

    def __str__(self):
        return str(self.day.strftime("%d/%m/%Y - %a"))

class MyDict(SortedDict):

    """a SortedDict that creates on the fly empty SortedDict as elements if not
    declared and that allows a sum with a list of empty elements (promoting to list)
    """

    def __getitem__(self, name):
        if name not in self.keys():
            super(MyDict, self).__setitem__(name, self.__class__())
        return super(MyDict, self).__getitem__(name)

    def __add__(a, b):
        """don't complain if the value is empty, just return added value"""
        if not a:
            return b
        return super(MyDict, self).__add__(a, b)

    def cls(self):
        return ''

    def to_html(self, user=None):
        return u''

    def __str__(self):
        if not self:
            return ''
        else:
            return super(MyDict, self).__str__()

    def __unicode__(self):
        if not self:
            return u''
        else:
            return super(MyDict, self).__unicode__()


class TransitSlot(object):

    """a class that holds info on transit: both real and missing
    It's not a Transit object in that it exists also when no transit is present
    and can hold more than 1 transit till the situation is not reviewd by users.
    Essentially it's the best representation of the single cell in the
    html table, so must be able to render a correct class, a and create links

    """

    def __init__(self, datetime, Id, profile_id, col, miss=False, blank=False, transit=None):
        self.datetime = [datetime]
        self.ids = {datetime: Id}
        self.profile_id = profile_id
        self.col = col
        self.transit = transit
        ## a holiday is a regular missing, don't show 'missing'
        self.miss = miss and (datetime.date() not in HOLIDAYS)
        self.blank = blank
        if self.miss and self.datetime[0] > datetime.now():
            self.miss = False
            self.blank = True
        self.extra_class = ''

    def __add__(a, b):
        ## Note! this only works if adding an element with 1 datetime!!!
        # self.datetime is a list!
        a.datetime.extend(b.datetime)
        a.ids[b.datetime[0]] = b.ids[b.datetime[0]]
        return a

    def to_html(self, user_id):  # __str__(self):
        #import pdb; pdb.set_trace()
        if self.miss:
            return self.link(self.datetime[0], user_id)
        elif self.blank:
            return u''
        else:
            return u" - ".join([self.link(x, user_id) for x in self.datetime])

    def link(self, datetime, user_id):
        opts = 'top=200, left=200, width=580, height=320, status=no, menubar=no, toolbar=no scrollbar=no'
        try:
            info = {
                'user': user_id,
                'odv': self.col,
                'date': self.datetime[0].strftime('%Y/%m/%d')
            }
        except:
            #import pdb
            #pdb.set_trace()
            raise
        link = '/timereport/transit/add/%(user)s/%(odv)s/%(date)s/?mode=short' % (info)
        onclick = "window.open('%s', '', '%s'); return false;" % (link, opts)
        if self.miss:
            return '<a href="" onclick="%s">%s</a>' % (onclick, _('Missing'))
        else:
            return """<a href="" onclick="window.open('/timereport/transit/edit/%s/', '', '%s'); return false;">%s</a>""" % (
                self.ids[datetime], opts.replace('320', '520'), datetime.strftime('%H:%M'))

    def cls(self):
        #import pdb; pdb.set_trace()
        if self.miss or len(self.datetime) > 1:
            return 'error'
        elif self.blank:
            return ''
        else:
            if self.transit and self.transit.reason.column == 'illness':
                return self.classes("illness")

            if self.transit and self.transit.reason.worked != 'w':
                if self.transit.approved == 2:
                    return self.classes("vacation-approved")
                else:
                    return self.classes("vacation")

            if self.transit and self.transit.status == 2:
                return self.classes("manual")

            if self.datetime[0].date() in HOLIDAYS:
                return self.classes("extra")
            try:
                self.official_time = get_official_time(self.profile_id, self.datetime[0], self.col)
            except:
                return self.classes("extra")

            try:
                delta = self.datetime[0] - datetime.combine(self.datetime[0], self.official_time)
            except TypeError:
                # print "TypeError in ", self.profile_id, self.datetime[0].strftime("%w
                # %a"), self.datetime[0], self.col
                return ''

            #import pdb; pdb.set_trace()
            delta_min = delta2min(delta)

            if self.col in (1, 3):
                if delta_min < 0:  # before official time
                    if delta_min < -ProfileDeltas[self.profile_id][0]:
                        return self.classes('longer')
                    else:
                        return ''
                else:  # late
                    if delta_min > ProfileDeltas[self.profile_id][1]:
                        return self.classes('shorter')
                    else:
                        return ''

            if self.col in (2, 4):
                if delta_min < 0:  # before official time
                    if delta_min < -ProfileDeltas[self.profile_id][0]:
                        return self.classes('shorter')
                    else:
                        return self.classes('')
                else:
                    if delta_min > ProfileDeltas[self.profile_id][1]:
                        return self.classes('longer')
                    else:
                        return self.classes('')

    def classes(self, html_class):
        return "%s %s" % (self.extra_class, html_class)

    def tip(self):
        if self.transit:
            if self.transit.iter_id:
                self.extra_class = 'help_text'
                return 'title="%s"' % Notes[self.transit.iter_id].description
        return ''

    def __repr__(self):
        return "<TransitSlot: %s (%s)>" % ([d.strftime('%x %X')
                                            for d in self.datetime], self.transit)

def get_official_time(profile_id, date, col):
    # detail has: time + duration!
    details = ProfileDayDetails[profile_id][date.isoweekday()]
    if col % 2:
        return details[col].time
    else:
        return details[col-1].time_out

def get_official_datetime(profile_id, day, col):
    date = day.day
    time = get_official_time(profile_id, date, col)
    return datetime.combine(date, time)

def get_transits(filter={}, queryset=None):
    """
    return a tuple (transits, days, users, Totals, PeriodTotals). Each is a dict
    suitable for looping ie:
    days: all days are present in the required range, even days in which
          no transit is registered
    transits: every cell has a TransitSlot object so that can be used
          easily in templates w/o checking existance
    """
    global Notes
    T = MyDict()
    Users = MyDict()
    Days = []
    Totals = MyDict()
    Notes = {}  # id of the iter is the key, the value is the iter
    PeriodTotals = MyDict()

    queryset = queryset.filter(**filter).select_related()
    for t in queryset.order_by('user', 'date_reference', 'reason', 'date_time'):
        day = t.date_reference
        Days += [day]
        T[t.user_id][day][t.odv] += TransitSlot(
            t.date_time, t.id, t.profile.id, t.odv, transit=t)
        if t.iter_id:
            if t.iter_id not in Notes:
                Notes[t.iter_id] = t.iter

    ## fill in data on users
    query = UserProfile.objects.filter(user__in=T.keys())
    for u in query.order_by('badge_num'):
        Users[u.user_id]['user'] = u.user
        Users[u.user_id]['profile_id'] = u.time_profile.id

    ## check all entries needed as per profile are present else add
    ## a TransitSlot that will translate to Alert / Missing
    DayList = all_days_as_list(filter, Days, )
    for user_id in T.keys():

        ## the following profile_id will be overwritten by the first
        ## transit, but is here in case the first day the user is missing
        profile_id = Users[user_id]['profile_id']
        period_total = PeriodTotal(user_id,)
        PeriodTotals[user_id] = period_total
        debug_profiles = ProfileDayDetails.keys()
        for day in DayList:

            total = Total(day, ProfileDayDetails[profile_id][day.day.isoweekday()])
            Totals[user_id][day.day] = total
            for col in (1, 2, 3, 4, 11, 12):

                if T[user_id][day.day][col]:

                    total.add(T[user_id][day.day][col].transit)
                    profile_id = T[user_id][day.day][col].profile_id

                else:
                    add_blank_or_missing_transit(T, profile_id, user_id, day, col)

            ## NOTE: period_total.add triggers the calculus of time!!
            period_total.add(total)
    return (T, DayList, Users, Totals, PeriodTotals)

def add_blank_or_missing_transit(T, profile_id, user_id, day, col):
    # no entry for this user/day/col, maybe just col 11, 12...
    # this is ok but I want to define a TransitSlot instance just
    # for template usage.

    blank = False
    miss = False
    try:
        ## next will fail if no official datetime is due
        date_time = get_official_datetime(profile_id, day, col)

        miss = True
        if day.is_holiday():
            blank = True

    except AttributeError as e:  # 'MyDict' object has no attribute 'time'
        ## it's ok not to have anything
        date_time = datetime.combine(day.day, time(0, 0))
        blank = True

    T[user_id][day.day][col] = TransitSlot(
        date_time, None, profile_id, col, miss=miss, blank=blank)


def all_days_as_list(filter, days):
    """if filter has date_time__gte / date_time__lte use those days
    else look in days to get the start/end period
    """
    #import pdb; pdb.set_trace()
    #from jungle.forms.fields import JungleDateField
    #f = JungleDateField
    if 'date_reference__gte' in filter:
        start = filter['date_reference__gte']
    else:
        try:
            start = days[0]
        except IndexError:
            return []

    if 'date_reference__lte' in filter:
        end = filter['date_reference__lte']
    else:
        try:
            end = days[-1]
        except IndexError:
            end = date.today()

    if start == end:
        return [Day(start)]
    if start > end:  # prevent infinite looping
        return []

    D = [Day(start)]
    last = Day(start)

    while True:
        last = Day(last.next())
        D += [last]
        if last.day == end:
            return D

def delta2min(delta):
    seconds = delta.days*86400+delta.seconds
    return seconds/60

ProfileDayDetails = TimetableProfile.get_details()
ProfileDeltas = TimetableProfile.get_deltas()
